@@ -15,7 +15,7 @@
 #include <QLoggingCategory>
 #include <QProcess>
 
-Q_LOGGING_CATEGORY(dockGlobalElementModelLog, "dde.shell.dock.taskmanager.dockglobalelementmodel")
+Q_LOGGING_CATEGORY(dockGlobalElementModelLog, "org.deepin.dde.shell.dock.taskmanager.dockglobalelementmodel")
 
 #include <algorithm>
 #include <tuple>
@@ -29,122 +29,171 @@ DockGlobalElementModel::DockGlobalElementModel(QAbstractItemModel *appsModel, Do
     , m_activeAppModel(activeAppModel)
 {
     connect(TaskManagerSettings::instance(), &TaskManagerSettings::dockedElementsChanged, this, &DockGlobalElementModel::loadDockedElements);
-    connect(m_appsModel, &QAbstractItemModel::rowsRemoved, this, [this](const QModelIndex &parent, int first, int last) {
-        Q_UNUSED(parent)
-        for (int i = first; i <= last; ++i) {
-            auto it = std::find_if(m_data.begin(), m_data.end(), [this, &i](auto data) {
-                return std::get<1>(data) == m_appsModel && std::get<2>(data) == i;
-            });
-            if (it != m_data.end()) {
-                auto pos = it - m_data.begin();
-                beginRemoveRows(QModelIndex(), pos, pos);
-                m_data.remove(pos);
-                endRemoveRows();
-            }
-        }
-        std::for_each(m_data.begin(), m_data.end(), [this, first, last](auto &data) {
-            if (std::get<1>(data) == m_appsModel && std::get<2>(data) >= first) {
-                data = std::make_tuple(std::get<0>(data), std::get<1>(data), std::get<2>(data) - ((last - first) + 1));
-            }
-        });
-    });
-
-    connect(m_activeAppModel, &QAbstractItemModel::rowsInserted, this, [this](const QModelIndex &parent, int first, int last) {
-        Q_UNUSED(parent)
-        for (int i = first; i <= last; ++i) {
-            auto index = m_activeAppModel->index(i, 0);
-            auto desktopId = index.data(TaskManager::DesktopIdRole).toString();
-
-            auto it = std::find_if(m_data.begin(), m_data.end(), [this, &desktopId](auto &data) {
-                return m_appsModel == std::get<1>(data) && desktopId == std::get<0>(data);
-            });
-
-            if (it != m_data.end()) {
-                *it = std::make_tuple(desktopId, m_activeAppModel, i);
-                auto pIndex = this->index(it - m_data.begin(), 0);
-                Q_EMIT dataChanged(pIndex, pIndex, {TaskManager::ActiveRole, TaskManager::AttentionRole, TaskManager::WindowsRole, TaskManager::MenusRole});
-
-            } else {
-                beginInsertRows(QModelIndex(), m_data.size(), m_data.size());
-                m_data.append(std::make_tuple(desktopId, m_activeAppModel, first));
-                endInsertRows();
-            }
-        }
-
-        std::for_each(m_data.begin(), m_data.end(), [this, first, last](auto &data) {
-            if (std::get<1>(data) == m_activeAppModel && std::get<2>(data) > first) {
-                data = std::make_tuple(std::get<0>(data), std::get<1>(data), std::get<2>(data) + ((last - first) + 1));
-            }
-        });
-    });
-
-    connect(m_activeAppModel, &QAbstractItemModel::rowsRemoved, this, [this](const QModelIndex &parent, int first, int last) {
-        Q_UNUSED(parent)
-        for (int i = first; i <= last; ++i) {
-            auto it = std::find_if(m_data.begin(), m_data.end(), [this, i](auto data) {
-                return std::get<1>(data) == m_activeAppModel && std::get<2>(data) == i;
-            });
-
-            if (it == m_data.end()) {
-                qWarning() << "failed to find a running apps on dock" << i;
-                continue;
-            }
-
-            auto pos = it - m_data.begin();
-            auto id = std::get<0>(*it);
-
-            auto oit = std::find_if(m_data.constBegin(), m_data.constEnd(), [this, &id, i](auto &data) {
-                return std::get<0>(data) == id && std::get<1>(data) == m_activeAppModel && std::get<2>(data) != i;
-            });
-
-            if (oit == m_data.constEnd() && m_dockedElements.contains(std::make_tuple("desktop", id))) {
-                auto pIndex = this->index(pos, 0);
-                auto res = m_appsModel->match(m_appsModel->index(0, 0), TaskManager::DesktopIdRole, id, 1, Qt::MatchExactly);
-                if (res.isEmpty()) {
+    connect(
+        m_appsModel,
+        &QAbstractItemModel::rowsRemoved,
+        this,
+        [this](const QModelIndex &parent, int first, int last) {
+            Q_UNUSED(parent)
+            for (int i = first; i <= last; ++i) {
+                auto it = std::find_if(m_data.begin(), m_data.end(), [this, &i](auto data) {
+                    return std::get<1>(data) == m_appsModel && std::get<2>(data) == i;
+                });
+                if (it != m_data.end()) {
+                    auto pos = it - m_data.begin();
                     beginRemoveRows(QModelIndex(), pos, pos);
                     m_data.remove(pos);
                     endRemoveRows();
-                } else {
-                    auto row = res.first().row();
-                    *it = std::make_tuple(id, m_appsModel, row);
-                    Q_EMIT dataChanged(pIndex, pIndex, {TaskManager::ActiveRole, TaskManager::AttentionRole, TaskManager::WindowsRole, TaskManager::MenusRole});
                 }
-            } else {
-                beginRemoveRows(QModelIndex(), pos, pos);
-                m_data.remove(pos);
-                endRemoveRows();
             }
-        }
-        std::for_each(m_data.begin(), m_data.end(), [this, first, last](auto &data) {
-            if (std::get<1>(data) == m_activeAppModel && std::get<2>(data) >= first) {
-                data = std::make_tuple(std::get<0>(data), std::get<1>(data), std::get<2>(data) - ((last - first) + 1));
-            }
-        });
-    });
-
-    connect(m_activeAppModel,
-            &QAbstractItemModel::dataChanged,
-            this,
-            [this](const QModelIndex &topLeft, const QModelIndex &bottomRight, const QList<int> &roles) {
-                int first = topLeft.row(), last = bottomRight.row();
-                for (int i = first; i <= last; i++) {
-                    auto it = std::find_if(m_data.constBegin(), m_data.constEnd(), [this, i](auto data) {
-                        return std::get<1>(data) == m_activeAppModel && std::get<2>(data) == i;
-                    });
-
-                    if (it == m_data.end())
-                        return;
-                    auto pos = it - m_data.constBegin();
-
-                    auto oldRoles = roles;
-                    auto desktopId = roles.indexOf(TaskManager::DesktopIdRole);
-                    auto identifyId = roles.indexOf(TaskManager::IdentityRole);
-                    if (desktopId != -1 || identifyId != -1) {
-                        oldRoles.append(TaskManager::ItemIdRole);
-                    }
-                    Q_EMIT dataChanged(index(pos, 0), index(pos, 0), oldRoles);
+            std::for_each(m_data.begin(), m_data.end(), [this, first, last](auto &data) {
+                if (std::get<1>(data) == m_appsModel && std::get<2>(data) >= first) {
+                    data = std::make_tuple(std::get<0>(data), std::get<1>(data), std::get<2>(data) - ((last - first) + 1));
                 }
             });
+        },
+        Qt::QueuedConnection);
+
+    connect(
+        m_activeAppModel,
+        &QAbstractItemModel::rowsInserted,
+        this,
+        [this](const QModelIndex &parent, int first, int last) {
+            Q_UNUSED(parent)
+            for (int i = first; i <= last; ++i) {
+                auto index = m_activeAppModel->index(i, 0);
+                auto desktopId = index.data(TaskManager::DesktopIdRole).toString();
+
+               if (desktopId.isEmpty())
+                    continue;
+                //将同一应用的窗口添加到一起 
+                // Find the first occurrence of this app in m_data (either docked item or existing window)
+                auto firstIt = std::find_if(m_data.begin(), m_data.end(), [&desktopId](const auto &data) {
+                    return std::get<0>(data) == desktopId;
+                });
+
+                if (firstIt == m_data.end()) {
+                    // No docked item or existing window yet, append to the end
+                    beginInsertRows(QModelIndex(), m_data.size(), m_data.size());
+                    m_data.append(std::make_tuple(desktopId, m_activeAppModel, i));
+                    endInsertRows();
+                    continue;
+                }
+
+                // If the first occurrence still comes from m_appsModel, this is the first window:
+                // reuse the docked position and turn it into a running window.
+                if (std::get<1>(*firstIt) == m_appsModel) {
+                    *firstIt = std::make_tuple(desktopId, m_activeAppModel, i);
+                    auto pIndex = this->index(firstIt - m_data.begin(), 0);
+                    Q_EMIT dataChanged(pIndex,
+                                       pIndex,
+                                       {TaskManager::ActiveRole,
+                                        TaskManager::AttentionRole,
+                                        TaskManager::WindowsRole,
+                                        TaskManager::MenusRole,
+                                        TaskManager::WinTitleRole});
+                    continue;
+                }
+
+                // There are already windows for this app: insert the new window
+                // right after the last existing one so that all windows stay together.
+                auto lastIt = firstIt;
+                while (lastIt + 1 != m_data.end() && std::get<0>(*(lastIt + 1)) == desktopId) {
+                    ++lastIt;
+                }
+
+                auto insertRow = (lastIt - m_data.begin()) + 1;
+                beginInsertRows(QModelIndex(), insertRow, insertRow);
+                m_data.insert(lastIt + 1, std::make_tuple(desktopId, m_activeAppModel, i));
+                endInsertRows();
+            }
+
+            std::for_each(m_data.begin(), m_data.end(), [this, first, last](auto &data) {
+                if (std::get<1>(data) == m_activeAppModel && std::get<2>(data) > first) {
+                    data = std::make_tuple(std::get<0>(data),
+                                           std::get<1>(data),
+                                           std::get<2>(data) + ((last - first) + 1));
+                }
+            });
+        },
+        Qt::QueuedConnection);
+
+    connect(
+        m_activeAppModel,
+        &QAbstractItemModel::rowsRemoved,
+        this,
+        [this](const QModelIndex &parent, int first, int last) {
+            Q_UNUSED(parent)
+            for (int i = first; i <= last; ++i) {
+                auto it = std::find_if(m_data.begin(), m_data.end(), [this, i](auto data) {
+                    return std::get<1>(data) == m_activeAppModel && std::get<2>(data) == i;
+                });
+
+                if (it == m_data.end()) {
+                    qWarning() << "failed to find a running apps on dock" << i;
+                    continue;
+                }
+
+                auto pos = it - m_data.begin();
+                auto id = std::get<0>(*it);
+
+                auto oit = std::find_if(m_data.constBegin(), m_data.constEnd(), [this, &id, i](auto &data) {
+                    return std::get<0>(data) == id && std::get<1>(data) == m_activeAppModel && std::get<2>(data) != i;
+                });
+
+                if (oit == m_data.constEnd() && m_dockedElements.contains(std::make_tuple("desktop", id))) {
+                    auto pIndex = this->index(pos, 0);
+                    auto res = m_appsModel->match(m_appsModel->index(0, 0), TaskManager::DesktopIdRole, id, 1, Qt::MatchExactly);
+                    if (res.isEmpty()) {
+                        beginRemoveRows(QModelIndex(), pos, pos);
+                        m_data.remove(pos);
+                        endRemoveRows();
+                    } else {
+                        auto row = res.first().row();
+                        *it = std::make_tuple(id, m_appsModel, row);
+                        Q_EMIT dataChanged(pIndex,
+                                           pIndex,
+                                           {TaskManager::ActiveRole, TaskManager::AttentionRole, TaskManager::WindowsRole, TaskManager::MenusRole, TaskManager::WinTitleRole});
+                    }
+                } else {
+                    beginRemoveRows(QModelIndex(), pos, pos);
+                    m_data.remove(pos);
+                    endRemoveRows();
+                }
+            }
+            std::for_each(m_data.begin(), m_data.end(), [this, first, last](auto &data) {
+                if (std::get<1>(data) == m_activeAppModel && std::get<2>(data) >= first) {
+                    data = std::make_tuple(std::get<0>(data), std::get<1>(data), std::get<2>(data) - ((last - first) + 1));
+                }
+            });
+        },
+        Qt::QueuedConnection);
+
+    connect(
+        m_activeAppModel,
+        &QAbstractItemModel::dataChanged,
+        this,
+        [this](const QModelIndex &topLeft, const QModelIndex &bottomRight, const QList<int> &roles) {
+            int first = topLeft.row(), last = bottomRight.row();
+            for (int i = first; i <= last; i++) {
+                auto it = std::find_if(m_data.constBegin(), m_data.constEnd(), [this, i](auto data) {
+                    return std::get<1>(data) == m_activeAppModel && std::get<2>(data) == i;
+                });
+
+                if (it == m_data.end())
+                    return;
+                auto pos = it - m_data.constBegin();
+
+                auto oldRoles = roles;
+                auto desktopId = roles.indexOf(TaskManager::DesktopIdRole);
+                auto identifyId = roles.indexOf(TaskManager::IdentityRole);
+                if (desktopId != -1 || identifyId != -1) {
+                    oldRoles.append(TaskManager::ItemIdRole);
+                }
+                Q_EMIT dataChanged(index(pos, 0), index(pos, 0), oldRoles);
+            }
+        },
+        Qt::QueuedConnection);
 
     QMetaObject::invokeMethod(this, &DockGlobalElementModel::loadDockedElements, Qt::QueuedConnection);
 }
@@ -284,7 +333,11 @@ QString DockGlobalElementModel::getMenus(const QModelIndex &index) const
         if (TaskManagerSettings::instance()->isAllowedForceQuit()) {
             menusArray.append(QJsonObject{{"id", DOCK_ACTION_FORCEQUIT}, {"name", tr("Force Quit")}});
         }
-        menusArray.append(QJsonObject{{"id", DOCK_ACTION_CLOSEALL}, {"name", tr("Close All")}});
+        if (TaskManagerSettings::instance()->isWindowSplit()) {
+            menusArray.append(QJsonObject{{"id", DOCK_ACTION_CLOSEWINDOW}, {"name", tr("Close")}});
+        } else {
+            menusArray.append(QJsonObject{{"id", DOCK_ACTION_CLOSEALL}, {"name", tr("Close All")}});
+        }
     }
 
     return QJsonDocument(menusArray).toJson();
@@ -340,7 +393,53 @@ void DockGlobalElementModel::requestActivate(const QModelIndex &index) const
         auto sourceIndex = sourceModel->index(sourceRow, 0);
         m_activeAppModel->requestActivate(sourceIndex);
     } else {
-        this->requestNewInstance(index, "");
+        if (auto interface = dynamic_cast<AbstractTaskManagerInterface*>(sourceModel)) {
+            auto sourceIndex = sourceModel->index(sourceRow, 0);
+            interface->requestNewInstance(sourceIndex, "");
+        }
+    }
+}
+
+void DockGlobalElementModel::requestNewInstance(const QModelIndex &index, const QString &action) const
+{
+    auto data = m_data.value(index.row());
+    auto id = std::get<0>(data);
+    auto sourceModel = std::get<1>(data);
+    auto sourceRow = std::get<2>(data);
+
+    // Handle special actions first (for both active and docked apps)
+    if (action == DOCK_ACTION_DOCK) {
+        TaskManagerSettings::instance()->toggleDockedElement(QStringLiteral("desktop/%1").arg(id));
+        return;
+    } else if (action == DOCK_ACTION_FORCEQUIT) {
+        requestClose(index, true);
+        return;
+    } else if (action == DOCK_ACTION_CLOSEWINDOW || action == DOCK_ACTION_CLOSEALL) {
+        requestClose(index, false);
+        return;
+    }
+
+    //应用自身处理的action
+    if (!action.isEmpty()) {
+        QProcess process;
+        process.setProcessChannelMode(QProcess::MergedChannels);
+        process.start("dde-am", {"--by-user", id, action});
+        process.waitForFinished();
+        return;
+    }
+
+    // Handle launch/activate (empty action)
+    if (sourceModel == m_activeAppModel) {
+        auto sourceIndex = sourceModel->index(sourceRow, 0);
+        m_activeAppModel->requestNewInstance(sourceIndex, action);
+    } else {
+        QString dbusPath = QStringLiteral("/org/desktopspec/ApplicationManager1/") + escapeToObjectPath(id);
+        using Application = org::desktopspec::ApplicationManager1::Application;
+        Application appInterface(QStringLiteral("org.desktopspec.ApplicationManager1"), dbusPath, QDBusConnection::sessionBus());
+
+        if (appInterface.isValid()) {
+            appInterface.Launch(QString(), QStringList(), QVariantMap());
+        }
     }
 }
 
@@ -363,26 +462,6 @@ void DockGlobalElementModel::requestOpenUrls(const QModelIndex &index, const QLi
     }
 }
 
-void DockGlobalElementModel::requestNewInstance(const QModelIndex &index, const QString &action) const
-{
-    if (action == DOCK_ACTION_DOCK) {
-        auto data = m_data.value(index.row());
-        auto id = std::get<0>(data);
-        TaskManagerSettings::instance()->toggleDockedElement(QStringLiteral("desktop/%1").arg(id));
-    } else if (action == DOCK_ACTION_FORCEQUIT) {
-        requestClose(index, true);
-    } else if (action == DOCK_ACTION_CLOSEALL) {
-        requestClose(index);
-    } else {
-        auto data = m_data.value(index.row());
-        auto id = std::get<0>(data);
-        QProcess process;
-        process.setProcessChannelMode(QProcess::MergedChannels);
-        process.start("dde-am", {"--by-user", id, action});
-        process.waitForFinished();
-        return;
-    }
-}
 void DockGlobalElementModel::requestClose(const QModelIndex &index, bool force) const
 {
     auto data = m_data.value(index.row());
@@ -398,11 +477,18 @@ void DockGlobalElementModel::requestClose(const QModelIndex &index, bool force)
 
     qWarning() << "unable to close app not running";
 }
-void DockGlobalElementModel::requestUpdateWindowGeometry(const QModelIndex &index, const QRect &geometry, QObject *delegate) const
+
+void DockGlobalElementModel::requestUpdateWindowIconGeometry(const QModelIndex &index, const QRect &geometry, QObject *delegate) const
 {
-    Q_UNUSED(index)
-    Q_UNUSED(geometry)
-    Q_UNUSED(delegate)
+    auto data = m_data.value(index.row());
+    auto id = std::get<0>(data);
+    auto sourceModel = std::get<1>(data);
+    auto sourceRow = std::get<2>(data);
+
+    if (sourceModel == m_activeAppModel) {
+        auto sourceIndex = sourceModel->index(sourceRow, 0);
+        m_activeAppModel->requestUpdateWindowIconGeometry(sourceIndex, geometry, delegate);
+    }
 }
 
 void DockGlobalElementModel::requestWindowsView(const QModelIndexList &indexes) const
